import {useEffect, useState} from 'react';
import './home.css';
import {
    Button, Dialog, DialogTrigger, makeStyles,mergeClasses, MenuItem, MenuList, tokens, Tooltip,
} from "@fluentui/react-components";
import {
    CloudAddFilled, DismissRegular
} from "@fluentui/react-icons";
import {Dial} from "../../api";
import {useToast} from "../../message";
import {Connection} from "../../interfaces/connection";
import {ConnectionCreate} from "../connection/new";

const useMenuListContainerStyles = makeStyles({
    container: {
        backgroundColor: tokens.colorNeutralBackground1,
        flex: 1,
        width: "100%",
        paddingTop: "4px",
        paddingBottom: "4px",
    },
    item: {
        display: 'flex',
        height: '100%',
        alignItems: 'center',
        flexDirection: 'row',
        fontSize: '15px',
        '& span': {
            display: 'flex',
            flexDirection: 'row',
            alignItems: 'center',
        },
        '&.active': {
            color: tokens.colorNeutralForeground2BrandHover,
        }
    },
    item_icon: {
        height: '100%',
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center',
        marginLeft: 'auto',
        border: 'none',
        background: 'transparent',
        "&:hover" : {
            color: tokens.colorNeutralForeground2BrandHover,
        }
    }
});


function Home() {
    const styles = useMenuListContainerStyles();
    const {dispatchMessage} = useToast();
    const [openCreate, setOpenCreate] = useState(false);
    const [connectionFilterKeywords, setConnectionFilterKeywords] = useState<string>('');
    const [connections, setConnections] = useState<Connection[]>([]);

    useEffect(() => {
        updateConnectionList().then()
    }, []);

    async function updateConnectionList() {
        const res = await Dial<{ list: Connection[] }>("/api/connection/list");
        dispatchMessage(res.status === 200 ? '获取连接列表成功' : res.msg, res.status === 200 ? 'success' : 'error');
        setConnections(res.status === 200 ? res.data.list : connections);
        setOpenCreate(false)
        return;
    }

    async function handleConnect(item: Connection) {
        console.log('[DEBUG] db click item =', item)
        for (const c of connections) {
            if (item.id === c.id && c.active) {
                console.log('[DEBUG] conn is already connected:', c)
                return
            }
        }

        let res = await Dial("/api/connection/connect", {id: item.id})
        if (res.status === 200) {
            dispatchMessage("连接成功", "success")
            setConnections(connections.map(one => {
                if (one.id === item.id) {
                    one.active = true
                }

                return one
            }))
        }
    }

    async function handleDisconnect(item: Connection) {
        let res = await Dial('/api/connection/disconnect', {id: item.id})
        if (res.status === 200) {
            setConnections(connections.map(c => {
                if (item.id === c.id) {
                    c.active = false
                }
                return c
            }))
        }
    }

    return (
        <div className="container">
            <div className="header">
                <Dialog
                    open={openCreate}
                    onOpenChange={(event, data) => setOpenCreate(data.open)}>
                    <DialogTrigger disableButtonEnhancement>
                        <Button appearance="primary" icon={<CloudAddFilled/>}>
                            新建连接
                        </Button>
                    </DialogTrigger>
                    <ConnectionCreate update={updateConnectionList}/>
                </Dialog>
            </div>
            <div className="body">
                <div className="body-connections">
                    <div className="body-connections-search">
                        <input className="body-connections-search-input" type={"text"} placeholder="搜索连接"
                               value={connectionFilterKeywords}
                               onChange={(e) => setConnectionFilterKeywords(e.target.value)}/>
                        <div className="body-connections-search-dismiss" onClick={() => {
                            setConnectionFilterKeywords('')
                        }}>
                            <DismissRegular/>
                        </div>
                    </div>
                    <div className="body-connections-list">
                        <div className={styles.container}>
                            <MenuList>
                                {connections.map(item => {
                                    return <MenuItem
                                        onDoubleClick={async () => {
                                            await handleConnect(item)
                                        }}
                                        className={item.active ? mergeClasses(styles.item, 'active') : styles.item}
                                        key={item.id}>
                                        {item.name}
                                        <Tooltip content="断开连接" relationship="label">
                                            <Button onClick={async () => {await handleDisconnect(item)}} size="small" className={styles.item_icon} icon={<DismissRegular />} />
                                        </Tooltip>
                                    </MenuItem>
                                })}
                            </MenuList>
                        </div>
                    </div>
                </div>
                <div className="body-content"></div>
            </div>
            <div className="footer"></div>
        </div>
    )
}

export default Home
